@@ -1,4 +1,4 @@
-/*
+ /*
  * This file is part of the GIRAFFE Pipeline
  * Copyright (C) 2002-2019 European Southern Observatory
  *
@@ -1287,29 +1287,29 @@ giraffe_select_flux_standard(const GiTable* catalog, const GiImage* spectra,
     	ra = cpl_propertylist_get_double(properties, GIALIAS_RADEG); //12.778222
     }
 
+
     if (cpl_propertylist_has(properties, GIALIAS_TARG_DELTA) ==1 ){
     	targ_delta = cpl_propertylist_get_double(properties, GIALIAS_TARG_DELTA); //-730439.400
     	dec_dd = (int) (targ_delta/10000);
     	dec_mm = (int) ((targ_delta-dec_dd*10000)/100);
     	dec_ss = targ_delta -(dec_dd*10000 + dec_mm*100);
 
-    	// DEC:
-    	if (dec_dd < 0.0) {
-    		dec = dec_dd - (dec_mm/60.0) - (dec_ss/3600.0);
-    	}
-    	else {
-    		dec = dec_dd + (dec_mm/60.0) + (dec_ss/3600.0);
-    	}
+    	//DEC
+    	dec = dec_dd + (dec_mm/60.0) + (dec_ss/3600.0);
     }
     else {
     	dec = cpl_propertylist_get_double(properties, GIALIAS_DECDEG); //-73.07761
     }
-
+    
 
     if (cpl_error_get_code() != CPL_ERROR_NONE) {
     	return NULL;
     }
-
+   
+    cpl_msg_info(_id, "TARG.ALPHA : %f ", targ_alpha);
+    cpl_msg_info(_id, "RA Used : %f ", ra);
+    cpl_msg_info(_id, "TARG.DELTA : %f ", targ_delta);
+    cpl_msg_info(_id, "DEC Used : %f ", dec);
 
     giraffe_error_pop();
 
