@@ -1978,6 +1978,8 @@ gistandard(cpl_parameterlist* config, cpl_frameset* set)
     flxstd = giraffe_table_new();
     status = giraffe_table_load(flxstd, filename, 1, NULL);
 
+    cpl_msg_info(_id, "Flux table loaded ...");
+
     if (status != 0) {
         cpl_msg_error(_id, "Cannot load flux standards catalog from "
                       "'%s'. Aborting ...", filename);
@@ -2001,12 +2003,15 @@ gistandard(cpl_parameterlist* config, cpl_frameset* set)
     }
 
 
+    fxcal_config = giraffe_fxcalibration_config_create(config);
+
    /*
     * Search the flux standard catalog for the observed flux standard
     * object.
     */
 
-    refflx = giraffe_select_flux_standard(flxstd, rebinning->spectra, 1.);
+    refflx = giraffe_select_flux_standard(flxstd, rebinning->spectra, fxcal_config->max_dist);
+
 
     if (refflx == NULL) {
         cpl_msg_error(_id, "No matching flux standard found in the "
@@ -2067,9 +2072,6 @@ gistandard(cpl_parameterlist* config, cpl_frameset* set)
         return 1;
     }
 
-
-    fxcal_config = giraffe_fxcalibration_config_create(config);
-
     response = giraffe_response_new();
 
     status = giraffe_calibrate_flux(response, rebinning, fibers, NULL,
