@@ -1,3 +1,8 @@
+# ========================================================================================================
+#
+# John Pritchard
+#
+# ========================================================================================================
 try:
   import numpy
   from pipeline_display import *
@@ -7,6 +12,7 @@ try:
   numpy.seterr(invalid='ignore')
   import giraf_der_snr
   from giSpectrumDisplay import *
+  #from pipeline_display import SpectrumDisplay as giSpectrumDisplay
   
   from astropy.coordinates import SkyCoord
   from astropy.table import Table
@@ -16,7 +22,7 @@ except ImportError:
   donothing=1
 
 def get_attrs(klass):
-  return [k for k in klass.__dict__.keys()
+  return [k for k in list(klass.__dict__.keys())
             if not k.startswith('__')
             and not k.endswith('__')]
 
@@ -77,8 +83,8 @@ class PlotableFlatWithLOCcentroid(object) :
       self.ff_loccentroid.readImage()
       self.fibres = self.ff_loccentroid.image.shape[1]
 
-      #Creating the points to plot based on the polynomail traces
-      self.ypos_fibres = range(self.ff_loccentroid.image.shape[0])
+      #Creating the points to plot based on the polynomial traces
+      self.ypos_fibres = list(range(self.ff_loccentroid.image.shape[0]))
       self.xpos_fibres = []
       for fibre in numpy.arange(self.fibres) :
         self.xpos_fibres.append(self.ff_loccentroid.all_hdu[0].data[:,fibre]+1.)
@@ -95,25 +101,32 @@ class PlotableFlatWithLOCcentroid(object) :
       subplot.autoscale(enable=False)
       ## EVEN number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 0)] :
-        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
-                     linestyle='solid',color='red')
+        subplot.plot(
+          self.xpos_fibres[fibre], self.ypos_fibres,
+          linestyle='solid',color='red'
+        )
       ## ODD number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 1)] :
-        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
-                     linestyle='solid',color='yellow')
+        subplot.plot(
+          self.xpos_fibres[fibre], self.ypos_fibres,
+          linestyle='solid',color='yellow'
+        )
       ## Label the subslits...
       for SS in numpy.unique(self.ff_loccentroid.all_hdu[1].data['SSN']):
-          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
-              xycoords='data', horizontalalignment='center', verticalalignment='top')
+        subplot.annotate(
+          str(SS),
+          xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
+          xycoords='data', horizontalalignment='center', verticalalignment='top'
+        )
       ## Label the fibres...
-      j=0
       for i in numpy.unique(self.ff_loccentroid.all_hdu[1].data['FPS']):
-        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,lss],lss),
-              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
-        j+=1
+        subplot.annotate(
+          str(i),
+          xy=(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['FPS'] == i)][0][lss],lss),
+          xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.
+        )
     ## A horizontal line at the sloc-start value...
-    subplot.plot([0,self.NAXIS1], [lss,lss],
-                 linestyle='solid', color='black', linewidth=2)
+    subplot.plot([0,self.NAXIS1], [lss,lss], linestyle='solid', color='black', linewidth=2)
 
   def setXsectRow(self, row):
       self.XsectRow=row
@@ -132,22 +145,30 @@ class PlotableFlatWithLOCcentroid(object) :
       subplot.autoscale(enable=False)
       ## EVEN number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 0)] :
-        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
-                     linestyle='solid',color='red')
+        subplot.axvline(
+          self.xpos_fibres[fibre][self.XsectRow],
+          linestyle='solid',color='red'
+        )
       ## ODD number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 1)] :
-        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
-                     linestyle='solid',color='yellow')
+        subplot.axvline(
+          self.xpos_fibres[fibre][self.XsectRow],
+          linestyle='solid',color='yellow'
+        )
       ## Label the subslits...
       for SS in numpy.unique(self.ff_loccentroid.all_hdu[1].data['SSN']):
-          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
-              xycoords='data', horizontalalignment='center', verticalalignment='top')
+        subplot.annotate(
+          str(SS),
+          xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
+          xycoords='data', horizontalalignment='center', verticalalignment='top'
+        )
       ## Label the fibres...
-      j=0
       for i in numpy.unique(self.ff_loccentroid.all_hdu[1].data['FPS']):
-        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,self.XsectRow],0),
-              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
-        j+=1
+        subplot.annotate(
+          str(i),
+          xy=(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['FPS'] == i)][0][self.XsectRow],0),
+          xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.
+        )
 
 ################################################################################
 
@@ -186,8 +207,8 @@ class PlotableFlatWithPSFcentroid(object) :
       self.ff_psfcentroid.readImage()
       self.fibres = self.ff_psfcentroid.image.shape[1]
 
-      #Creating the points to plot based on the polynomail traces
-      self.ypos_fibres = range(self.ff_psfcentroid.image.shape[0])
+      #Creating the points to plot based on the polynomial traces
+      self.ypos_fibres = list(range(self.ff_psfcentroid.image.shape[0]))
       self.xpos_fibres = []
       for fibre in range(self.fibres) :
         self.xpos_fibres.append(self.ff_psfcentroid.all_hdu[0].data[:,fibre]+1.)
@@ -204,22 +225,30 @@ class PlotableFlatWithPSFcentroid(object) :
       subplot.autoscale(enable=False)
       ## EVEN number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 0)] :
-        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
-                     linestyle='solid',color='blue')
+        subplot.plot(
+          self.xpos_fibres[fibre], self.ypos_fibres,
+          linestyle='solid',color='blue'
+        )
       ## ODD number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 1)] :
-        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
-                     linestyle='solid',color='green')
+        subplot.plot(
+          self.xpos_fibres[fibre], self.ypos_fibres,
+          linestyle='solid',color='green'
+        )
       ## Label the subslits...
       for SS in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['SSN']):
-          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
-              xycoords='data', horizontalalignment='center', verticalalignment='top')
+          subplot.annotate(
+            str(SS),
+            xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
+            xycoords='data', horizontalalignment='center', verticalalignment='top'
+          )
       ## Label the fibres...
-      j=0
       for i in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['FPS']):
-        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,lss],lss),
-              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
-        j+=1
+        subplot.annotate(
+          str(i),
+          xy=(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['FPS'] == i)][0][lss],lss),
+          xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.
+        )
     ## A horizontal line at the sloc-start value...
     subplot.plot([0,self.NAXIS1], [lss,lss],
                  linestyle='solid', color='black', linewidth=2)
@@ -241,22 +270,30 @@ class PlotableFlatWithPSFcentroid(object) :
       subplot.autoscale(enable=False)
       ## EVEN number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 0)] :
-        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
-                     linestyle='solid',color='blue')
+        subplot.axvline(
+          self.xpos_fibres[fibre][self.XsectRow],
+          linestyle='solid',color='blue'
+        )
       ## ODD number subslits...
       for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 1)] :
-        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
-                     linestyle='solid',color='green')
+        subplot.axvline(
+          self.xpos_fibres[fibre][self.XsectRow],
+          linestyle='solid',color='green'
+        )
       ## Label the subslits...
       for SS in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['SSN']):
-          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
-              xycoords='data', horizontalalignment='center', verticalalignment='top')
+        subplot.annotate(
+          str(SS),
+          xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
+          xycoords='data', horizontalalignment='center', verticalalignment='top'
+        )
       ## Label the fibres...
-      j=0
       for i in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['FPS']):
-        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,self.XsectRow],0),
-              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
-        j+=1
+        subplot.annotate(
+          str(i),
+          xy=(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['FPS'] == i)][0][self.XsectRow],0),
+          xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.
+        )
 
 
 ################################################################################
@@ -309,7 +346,7 @@ class PlotableExtractedScience :
         self.PLparams = {}
         for f in fits_extractedscience :
             HDUlist=pyfits.open(f.name)
-            CRCM=HDUlist[0].header['CMETHOD']
+            CRCM=HDUlist[0].header.get('CMETHOD','NONE')
             self.INS_MODE=HDUlist[0].header['HIERARCH ESO INS MODE']
             self.RADEC=SkyCoord(HDUlist[0].header['RA'],HDUlist[0].header['DEC'], unit=(u.deg, u.deg), frame="icrs")
             self.extractedscience[CRCM] = PipelineProduct(f)
@@ -325,7 +362,7 @@ class PlotableExtractedScience :
             self.FIBER_SETUPs[CRCM]["DEC"].unit=u.deg
         for f in fits_extractederrors :
             HDUlist=pyfits.open(f.name)
-            CRCM=HDUlist[0].header['CMETHOD']
+            CRCM=HDUlist[0].header.get('CMETHOD','NONE')
             self.extractederrors[CRCM]  = PipelineProduct(f)
             HDUlist.close()
         self.spectrumdisplay   = giSpectrumDisplay()
@@ -371,9 +408,9 @@ class PlotableExtractedScience :
         self.isARG  = {}
         self.wave   = {}
         self.wave   = {}
-        for CRCM in self.extractederrors.keys() :
+        for CRCM in list(self.extractederrors.keys()) :
             self.extractederrors[CRCM].readImage()
-        for CRCM in self.extractedscience.keys() :
+        for CRCM in list(self.extractedscience.keys()) :
             self.extractedscience[CRCM].readImage()
             self.nobj[CRCM]   = self.extractedscience[CRCM].image.shape[1]
             self.crpix[CRCM]  = self.extractedscience[CRCM].readKeyword('CRPIX2', 0)
@@ -455,8 +492,8 @@ class PlotableExtractedScience :
         self.MED_lineSky=self.spectrumdisplay.overplot(subplot, [], [], errorflux=[], color = 'r')
         
     def errVerts(self, wave, flux, err):
-        vx=numpy.concatenate([wave,numpy.flip(wave,0)])
-        vy=numpy.concatenate([flux-err,numpy.flip(flux+err,0)])
+        vx=numpy.concatenate([wave,numpy.flipud(wave)])
+        vy=numpy.concatenate([flux-err,numpy.flipud(flux+err)])
         return [list(zip(vx,vy))]
         
 
@@ -479,7 +516,7 @@ class PlotableExtractedScience :
 
     def setSNRarr(self) :
         self.SNRarr={}
-        for CRCM in self.extractedscience.keys() :
+        for CRCM in list(self.extractedscience.keys()) :
             self.SNRarr[CRCM]=numpy.zeros(self.nobj[CRCM])
             if self.nobj[CRCM] == 1:
                 self.fib_id = 1
@@ -487,18 +524,18 @@ class PlotableExtractedScience :
                 self.SNRarr[CRCM][obj] = giraf_der_snr.DER_SNR(self.extractedscience[CRCM].image[:,obj])
 
     def selectHighestSNR(self, CRCM='NONE'):
-        self.objectSkyBySNR=numpy.append(self.sciObjectList[numpy.flip(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")]),0)],
-                                         self.skyList[numpy.flip(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]),0)])
+        self.objectSkyBySNR=numpy.append(self.sciObjectList[numpy.flipud(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")]))],
+                                         self.skyList[numpy.flipud(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]))])
         self.objectSkyByName_ind=numpy.append(numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")][numpy.argsort(self.sciObjectList)],
                                               numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")][numpy.argsort(self.skyList)])
-        self.objectSkyBySNR_ind=numpy.append(numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")][numpy.flip(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")]),0)],
-                                             numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")][numpy.flip(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]),0)])
+        self.objectSkyBySNR_ind=numpy.append(numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")][numpy.flipud(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] != "S")]))],
+                                             numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")][numpy.flipud(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]))])
 
         # Select Highest SNR
         if(self.fib_id == -1) :
             self.fib_id = numpy.argsort(self.SNRarr[CRCM])[-1]+1
 
-        self.skyBySNR_ind=numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")][numpy.flip(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]),0)]
+        self.skyBySNR_ind=numpy.arange(self.nobj[CRCM])[numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")][numpy.flipud(numpy.argsort(self.SNRarr[CRCM][numpy.where(self.FIBER_SETUPs[CRCM]['TYPE'][:] == "S")]))]
         self.skyListSNR=self.SNRarr[CRCM][self.skyBySNR_ind]
 
     def setFluxSelected(self) :
@@ -508,7 +545,7 @@ class PlotableExtractedScience :
         self.FPS = {}
         self.SSN = {}
         self.OBJECT_SNR = {}
-        for CRCM in self.extractedscience.keys() :
+        for CRCM in list(self.extractedscience.keys()) :
             self.flux[CRCM] = self.extractedscience[CRCM].image[:,self.fib_id-1]
             self.err[CRCM] = self.extractederrors[CRCM].image[:,self.fib_id-1]
             self.OBJECT[CRCM] = self.FIBER_SETUPs[CRCM]['OBJECT'][self.fib_id-1]
@@ -530,7 +567,7 @@ class PlotableExtractedScience :
         ## CRCM = NONE
         if self.flux is None :
             return
-        if 'NONE' in self.flux.keys() :
+        if 'NONE' in list(self.flux.keys()) :
             if self.NoCRC_displaySignal :
                 self.lineSignal[0].set_data(self.wave['NONE'], self.flux['NONE'])
                 if self.NoCRC_displayErr :
@@ -563,7 +600,7 @@ class PlotableExtractedScience :
                 self.lineSignalMinusSky[1].set_verts([])
         # --------------------------------------------------------------------------------
         ## CRCM = astroSCRAPPY
-        if 'ASTROSCRAPPY' in self.flux.keys() :
+        if 'ASTROSCRAPPY' in list(self.flux.keys()) :
             if self.AS_displaySignal :
                 self.AS_lineSignal[0].set_data(self.wave['ASTROSCRAPPY'], self.flux['ASTROSCRAPPY'])
                 if self.AS_displayErr :
@@ -596,7 +633,7 @@ class PlotableExtractedScience :
                 self.AS_lineSignalMinusSky[1].set_verts([])
         # --------------------------------------------------------------------------------
         ## CRCM = PyCosmic
-        if 'PYCOSMIC' in self.flux.keys() :
+        if 'PYCOSMIC' in list(self.flux.keys()) :
             if self.PC_displaySignal :
                 self.PC_lineSignal[0].set_data(self.wave['PYCOSMIC'], self.flux['PYCOSMIC'])
                 if self.PC_displayErr :
@@ -628,7 +665,7 @@ class PlotableExtractedScience :
                 self.PC_lineSignalMinusSky[0].set_data([],[])
                 self.PC_lineSignalMinusSky[1].set_verts([])
         ## CRCM = Median
-        if 'MEDIAN' in self.flux.keys() :
+        if 'MEDIAN' in list(self.flux.keys()) :
             if self.MED_displaySignal :
                 self.MED_lineSignal[0].set_data(self.wave['MEDIAN'], self.flux['MEDIAN'])
                 if self.MED_displayErr :
@@ -664,7 +701,7 @@ class PlotableExtractedScience :
 
     def computeSky(self, applyTo, method, statistic, numFibs=None ):
         self.sky={}
-        for CRCM in self.extractedscience.keys() :
+        for CRCM in list(self.extractedscience.keys()) :
             self.sky[CRCM] = self.computeSkyCRCM(applyTo, method, statistic, numFibs=numFibs, CRCM=CRCM)
         return self.sky
 
@@ -748,29 +785,29 @@ class PlotableExtractedScience :
                 cs_fibrange = [self.fib_id-1,]
                 cs_OneSkyValue=True
             else :
-                cs_fibrange = range(self.nobj[CRCM])
+                cs_fibrange = list(range(self.nobj[CRCM]))
                 if self.isARG[CRCM] :
                     tc=Table([[self.RADEC.ra.deg,],[self.RADEC.dec.deg,]], names=("RA","DEC"))
                 else:
                     tc=self.FIBER_SETUPs[CRCM]
                 otList = searchInCat(tc, self.FIBER_SETUPs[CRCM][self.skyBySNR_ind][self.skyFibIsSelected][self.skyRejected], rad=[0.,30.*60.], maxsources=numFibs)
         elif method == "Improves S/N" :
-            print("method = %s Not yet implemented" %(method))
+            print(("method = %s Not yet implemented" %(method)))
         else :
-            print("Error, method = %s not supported" %(method))
+            print(("Error, method = %s not supported" %(method)))
             return sky
 
         if applyTo == "All" :
             # Apply to all science spectra
             pass
         elif applyTo == "One" :
-            print("applyTo = %s Not yet implemented" %(applyTo))
+            print(("applyTo = %s Not yet implemented" %(applyTo)))
             # Apply to only the selected science spectra
         else :
-            print("Error, appyTo = %s not supported" %(appyTo))
+            print(("Error, appyTo = %s not supported" %(appyTo)))
             return sky
 
-        if not self.sky is None and CRCM in self.sky.keys() :
+        if not self.sky is None and CRCM in list(self.sky.keys()) :
             sky=self.sky[CRCM]
         else :
             sky=numpy.array([numpy.zeros(numpy.shape(self.extractedscience[CRCM].image)),numpy.zeros(numpy.shape(self.extractederrors[CRCM].image)),])
@@ -797,11 +834,11 @@ class PlotableExtractedScience :
                   numpy.mean(self.extractederrors[CRCM].image[:,cs_skyfibs],1),
                 ])
             elif statistic.lower() == "mode" :
-                print("statistic = %s Not yet implemented" %(statistic))
+                print(("statistic = %s Not yet implemented" %(statistic)))
                 ## If ever to implement, probably best to use scipy.statsi.mode(),see http://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.mode.html#scipy.stats.mode
                 ## and https://stackoverflow.com/questions/16330831/most-efficient-way-to-find-mode-in-numpy-array
             else :
-                print("Error, statistic = %s not supported" %(statistic))
+                print(("Error, statistic = %s not supported" %(statistic)))
                 sky=None
                 return sky
 
